@@ -291,36 +291,39 @@ static void mmkeys_init (MmKeys *object)
 	GdkScreen *screen;
 	GdkWindow *root;
 	gint i, j;
-	int keycode = 0;
+//	int keycode = 0;
 	int anyKeybindsFailed = FALSE;
 	int anyDuplicatesFound = FALSE;
 
 	display = gdk_display_get_default ();
 
 
-	/** Play Pause */
+	/** Play Pause 
 	keycode = XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPlay);
 	keycodes[MM_PLAYPAUSE] = cfg_get_single_value_as_int_with_default(config, "Keybindings", keynames[MM_PLAYPAUSE], keycode);
 	masks[MM_PLAYPAUSE] = cfg_get_single_value_as_int_with_default(config, "Keymasks", keynames[MM_PLAYPAUSE], 0);
+    */
 	/**
 	 * Previous
-	 */
+	 
 	keycode = XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioPrev);
 	keycodes[MM_PREV] = cfg_get_single_value_as_int_with_default(config, "Keybindings", keynames[MM_PREV], keycode);
 	masks[MM_PREV] = cfg_get_single_value_as_int_with_default(config, "Keymasks", keynames[MM_PREV], 0);
+    */
 	/**
 	 * Next 
-	 */
+	 *
 	keycode = XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioNext);
 	keycodes[MM_NEXT] = cfg_get_single_value_as_int_with_default(config, "Keybindings", keynames[MM_NEXT], keycode);
 	masks[MM_NEXT] = cfg_get_single_value_as_int_with_default(config, "Keymasks", keynames[MM_NEXT], 0);
+    */
 	/**
 	 * Stop
-	 */
+	 *
 	keycode = XKeysymToKeycode (GDK_DISPLAY (), XF86XK_AudioStop);
 	keycodes[MM_STOP] = cfg_get_single_value_as_int_with_default(config, "Keybindings", keynames[MM_STOP], keycode);
 	masks[MM_STOP] = cfg_get_single_value_as_int_with_default(config, "Keymasks", keynames[MM_STOP], 0);
-
+*/
 	for(i=0;i<LAST_SIGNAL;i++)
 	{
 		keycodes[i] = cfg_get_single_value_as_int_with_default(config, "Keybindings", keynames[i], 0);
@@ -363,6 +366,7 @@ static void mmkeys_init (MmKeys *object)
 
 	if (anyKeybindsFailed)
 	{
+        gchar *temp;
 		GString *message = g_string_new (_("Could not grab the following multimedia keys:\n\n"));
 		for (i=0;i<LAST_SIGNAL;i++)
 		{
@@ -379,14 +383,17 @@ static void mmkeys_init (MmKeys *object)
 		}
 		g_string_append( message,
 			_("\nEnsure that your window manager (or other applications) have not already bound this key for some other function, then restart gmpc." ));
-		show_error_message (message->str, TRUE);
+        temp = g_markup_escape_text(message->str, message->len);
+        playlist3_show_error_message(temp, ERROR_WARNING);
+        g_free(temp);
 		g_string_free (message, TRUE);
 	}
 
 	if (anyDuplicatesFound)
 	{
-		show_error_message(_("Duplicate mapping(s) detected\n\n"
-				"Some duplicate multimedia key mappings were detected, and disabled.  Please revisit the preferences and ensure your settings are now correct."), TRUE );
+        playlist3_show_error_message(_("Duplicate mapping(s) detected\n\n"
+				"Some duplicate multimedia key mappings were detected, and disabled.  Please revisit the preferences and ensure your settings are now correct."),     
+            ERROR_WARNING);
 	}
 }
 
@@ -678,8 +685,16 @@ void mmkeys_pref_destroy(GtkWidget *container)
 void mmkeys_pref_construct(GtkWidget *container)
 {
 	gchar *path = gmpc_get_full_glade_path("preferences-mmkeys.ui");
+    GError *error = NULL;
 	mmkeys_pref_xml = gtk_builder_new();
-    gtk_builder_add_from_file(mmkeys_pref_xml, path, NULL);
+    gtk_builder_add_from_file(mmkeys_pref_xml, path, &error);
+    if(error)
+    {
+        debug_printf(DEBUG_ERROR, "Problems loading ui: %s\n", error->message);
+        g_error_free(error);
+        g_object_unref(mmkeys_pref_xml);
+        return;
+    }
 	q_free(path);
 	if(mmkeys_pref_xml)
 	{
