@@ -1,22 +1,23 @@
-/*
- * Copyright (C) 2004 Lee Willis <lee@leewillis.co.uk>
- *    Borrowed heavily from code by Jan Arne Petersen <jpetersen@uni-bonn.de>
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
+/* Gnome Music Player Client (GMPC)
+ * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Borrowed from Lee Willis <lee@leewillis.co.uk> that
+ * Borrowed heavily from code by Jan Arne Petersen <jpetersen@uni-bonn.de>
+ * This projects' homepage is: http://gmpc.wikia.com/
+ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- */
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
 
 #include <X11/Xlib.h>
 #include <X11/XF86keysym.h>
@@ -52,7 +53,7 @@ struct _MmKeysClass
 MmKeys *mmkeys_new      (void);
 
 void grab_key(int key, int keycode, unsigned int mask);
-gmpcPlugin mmkeys_plug;
+extern gmpcPlugin mmkeys_plug;
 #endif /* __MM_KEYS_H */
 
 
